use std::collections::HashMap;
use std::io::Error;
use serde::{Serialize, Deserialize};

use dioxus::signals::Signal;
use dioxus::prelude::use_context;
use serde_json::Value;

#[derive(PartialEq, Clone, Serialize, Deserialize)]
pub enum Language {
    PlPL,
    EnGB,
    EnUS,
}

impl Language {
    fn translations(&self) -> &'static str {
        match self {
            Language::PlPL => include_str!("pl-PL.json"),
            Language::EnGB => include_str!("en-GB.json"),
            Language::EnUS => include_str!("en-US.json"),
        }
    }
}

impl From<&str> for Language {
    fn from(lang: &str) -> Self {
        match lang {
            "pl-PL" => Language::PlPL,
            "en-GB" => Language::EnGB,
            "en-US" => Language::EnUS,
            _ => Language::EnUS,
        }
    }
}

#[derive(PartialEq, Clone)]
pub struct LanguageManager {
    translations: Value
}

impl LanguageManager {
    pub fn new(language: Language) -> Self {
        let translations =
            load_translation_file(&language).unwrap_or_else(|_| Value::Null);

        Self { translations }
    }

    pub fn set(&mut self, lang: Language) {
        self.translations = load_translation_file(&lang).unwrap_or_else(|_| Value::Null);
    }

    fn get_nested_value<'a>(obj: &'a Value, path: &str) -> Option<&'a Value> {
        path.split('.').fold(Some(obj), |acc, key| acc?.get(key))
    }

    pub fn raw(&self, key: &str) -> Option<&Value> {
        Self::get_nested_value(&self.translations, key)
    }

    pub fn tr(&self, key: &str, map: Option<HashMap<String, String>>) -> String {
        let raw_value = self.raw(key).and_then(|v| v.as_str()).unwrap_or(key);
        let mut result = raw_value.to_string();

        if let Some(replacements) = map {
            for (placeholder, replacement) in replacements {
                let token = format!("{{%{}%}}", placeholder);
                result = result.replace(&token, &replacement);
            }
        }

        result
    }
}

fn load_translation_file(lang: &Language) -> Result<Value, Error> {
    let json: Value = serde_json::from_str(&lang.translations())?;

    Ok(json)
}

pub fn use_language() -> LanguageManager {
    let language = use_context::<Signal<Language>>();
    let mut handler = use_context::<LanguageManager>();
    
    handler.set(language());
    handler
}
